"""
Main entrypoint when MQTT IO is invoked as `python -m mqtt_io`.
"""

import argparse
import logging.config
import sys

from .config import load_main_config
from .exceptions import ConfigValidationFailed
from .server import MqttIo


def main() -> None:
    """
    Main entrypoint function.
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("config")
    args = parser.parse_args()

    # Load, validate and normalise config, or quit.
    try:
        config = load_main_config(args.config)
    except ConfigValidationFailed as exc:
        print(str(exc), file=sys.stderr)
        sys.exit(1)

    if config["logging"]:
        logging.config.dictConfig(config["logging"])

    mqtt_gpio = MqttIo(config)
    mqtt_gpio.run()


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        print("")



"""
Home Assistant MQTT Discovery features.

https://www.home-assistant.io/docs/mqtt/discovery/
"""

import json
import logging
from typing import Any, Dict

from .constants import INPUT_TOPIC, OUTPUT_TOPIC, SENSOR_TOPIC, SET_SUFFIX
from .mqtt import MQTTClientOptions, MQTTMessageSend
from .types import ConfigType

_LOG = logging.getLogger(__name__)


def get_common_config(
    mqtt_config: ConfigType, mqtt_options: MQTTClientOptions
) -> Dict[str, Any]:
    """
    Return config that's common across all HQ discovery announcements.
    """
    return dict(
        availability_topic="/".join(
            (mqtt_config["topic_prefix"], mqtt_config["status_topic"])
        ),
        payload_available=mqtt_config["status_payload_running"],
        payload_not_available=mqtt_config["status_payload_dead"],
        device=dict(
            manufacturer="MQTT IO",
            identifiers=["mqtt-io", mqtt_options.client_id],
            name=mqtt_config["discovery_name"],
        ),
    )


def hass_announce_digital_input(
    in_conf: ConfigType, mqtt_config: ConfigType, mqtt_options: MQTTClientOptions
) -> MQTTMessageSend:
    """
    Create a message which announces digital input as binary_sensor to Home Assistant.
    """
    name: str = in_conf["name"]
    disco_prefix: str = mqtt_config["discovery_prefix"]
    sensor_config = get_common_config(mqtt_config, mqtt_options)
    sensor_config.update(
        dict(
            name=name,
            unique_id=f"{mqtt_options.client_id}_{in_conf['module']}_input_{name}",
            state_topic="/".join((mqtt_config["topic_prefix"], INPUT_TOPIC, name)),
            payload_on=in_conf["on_payload"],
            payload_off=in_conf["off_payload"],
        )
    )
    return MQTTMessageSend(
        "/".join((disco_prefix, "binary_sensor", mqtt_options.client_id, name, "config")),
        json.dumps(sensor_config).encode("utf8"),
        retain=True,
    )


def hass_announce_digital_output(
    out_conf: ConfigType,
    mqtt_config: ConfigType,
    mqtt_options: MQTTClientOptions,
) -> MQTTMessageSend:
    """
    Create a message which announces digital output as switch to Home Assistant.
    """
    name: str = out_conf["name"]
    prefix: str = mqtt_config["topic_prefix"]
    disco_prefix: str = mqtt_config["discovery_prefix"]
    switch_config = get_common_config(mqtt_config, mqtt_options)
    switch_config.update(
        dict(
            name=name,
            unique_id=f"{mqtt_options.client_id}_{out_conf['module']}_output_{name}",
            state_topic="/".join((prefix, OUTPUT_TOPIC, name)),
            command_topic="/".join((prefix, OUTPUT_TOPIC, name, SET_SUFFIX)),
            payload_on=out_conf["on_payload"],
            payload_off=out_conf["off_payload"],
        )
    )
    return MQTTMessageSend(
        "/".join((disco_prefix, "switch", mqtt_options.client_id, name, "config")),
        json.dumps(switch_config).encode("utf8"),
        retain=True,
    )


def hass_announce_sensor_input(
    sens_conf: ConfigType,
    mqtt_config: ConfigType,
    mqtt_options: MQTTClientOptions,
) -> MQTTMessageSend:
    """
    Create a message which announces digital output as sensor to Home Assistant.
    """
    name: str = sens_conf["name"]
    prefix: str = mqtt_config["topic_prefix"]
    disco_prefix: str = mqtt_config["discovery_prefix"]
    expire_after: int = sens_conf.get("expire_after", sens_conf["interval"] * 2 + 5)
    sensor_config = get_common_config(mqtt_config, mqtt_options)
    sensor_config.update(
        dict(
            name=name,
            unique_id=f"{mqtt_options.client_id}_{sens_conf['module']}_sensor_{name}",
            state_topic="/".join((prefix, SENSOR_TOPIC, name)),
            expire_after=expire_after,
        )
    )
    if "unit_of_measurement" in sens_conf:
        sensor_config["unit_of_measurement"] = sens_conf["unit_of_measurement"]
    return MQTTMessageSend(
        "/".join((disco_prefix, "sensor", mqtt_options.client_id, name, "config")),
        json.dumps(sensor_config).encode("utf8"),
        retain=True,
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mqtt_io',
 'mqtt_io.config',
 'mqtt_io.config.validation',
 'mqtt_io.modules',
 'mqtt_io.modules.gpio',
 'mqtt_io.modules.sensor',
 'mqtt_io.modules.stream',
 'mqtt_io.mqtt',
 'mqtt_io.tests.features',
 'mqtt_io.tests.features.steps']

package_data = \
{'': ['*']}

install_requires = \
['Cerberus>=1.3.2,<2.0.0',
 'PyYAML>=5.3.1',
 'asyncio-mqtt>=0.8.1,<0.9.0',
 'typing-extensions>=3.7.4,<4.0.0']

extras_require = \
{':python_version >= "3.6" and python_version < "3.7"': ['dataclasses>=0.8,<0.9']}

setup_kwargs = {
    'name': 'mqtt-io',
    'version': '2.0.0b7',
    'description': 'Expose GPIO modules (Raspberry Pi, Beaglebone, PCF8754, PiFace2 etc.), digital sensors (LM75 etc.) and serial streams to an MQTT server for remote control and monitoring.',
    'long_description': None,
    'author': 'Ellis Percival',
    'author_email': 'mqtt-io@failcode.co.uk',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
